@@ -1,5 +1,6 @@
 /* This file is part of GNU cflow
-   Copyright (C) 1997, 2005, 2007, 2009, 2010, 2011 Sergey Poznyakoff
+   Copyright (C) 1997, 2005, 2007, 2009-2011, 2014-2016 Sergey
+   Poznyakoff
  
    GNU cflow is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -11,10 +12,8 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public
-   License along with GNU cflow; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   You should have received a copy of the GNU General Public License
+   along with this program.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <cflow.h>
 #include <argp.h>
@@ -262,10 +261,9 @@ symbol_override(const char *str)
      Symbol *sp;
      
      ptr = strchr(str, ':');
-     if (!ptr) {
-	  error(0, 0, _("%s: no symbol type supplied"), str);
-	  return;
-     } else {
+     if (!ptr)
+	  error(EX_USAGE, 0, _("%s: no symbol type supplied"), str);
+     else {
 	  name = strndup(str, ptr - str);
 	  if (ptr[1] == '=') {
 	       Symbol *alias = lookup(ptr+2);
@@ -283,10 +281,8 @@ symbol_override(const char *str)
 	       sp->flag = symbol_alias;
 	  } else {
 	       int type = find_option_type(symbol_optype, ptr+1, 0);
-	       if (type == 0) {
-		    error(0, 0, _("unknown symbol type: %s"), ptr+1);
-		    return;
-	       }
+	       if (type == 0)
+		    error(EX_USAGE, 0, _("unknown symbol type: %s"), ptr+1);
 	       sp = install(name, INSTALL_OVERWRITE);
 	       sp->type = SymToken;
 	       sp->token_type = type;
@@ -312,7 +308,7 @@ set_print_option(char *str)
      
      opt = find_option_type(print_optype, str, 0);
      if (opt == 0) {
-	  error(0, 0, _("unknown print option: %s"), str);
+	  error(EX_USAGE, 0, _("unknown print option: %s"), str);
 	  return;
      }
      print_option |= opt;
@@ -434,19 +430,16 @@ parse_level_string(const char *str, char **return_ptr)
 	       c = p[-1];
 	       for (i = 1; i < num; i++) {
 		    *p++ = c;
-		    if (*p == 0) {
-			 error(1, 0, _("level indent string is too long"));
-			 return;
-		    }
+		    if (*p == 0)
+			 error(EX_USAGE, 0,
+			       _("level indent string is too long"));
 	       }
 	       break;
 	  default:
 	  copy:
 	       *p++ = *str++;
-	       if (*p == 0) {
-		    error(1, 0, _("level indent string is too long"));
-		    return;
-	       }
+	       if (*p == 0)
+		    error(EX_USAGE, 0, _("level indent string is too long"));
 	  }
      }
      *p = 0;
@@ -471,10 +464,8 @@ set_level_indent(const char *str)
      
      p = str;
      while (*p != '=') {
-	  if (*p == 0) {
-	       error(1, 0, _("level-indent syntax"));
-	       return;
-	  }
+	  if (*p == 0)
+	       error(EX_USAGE, 0, _("level-indent syntax"));
 	  p++;
      }
      ++p;
@@ -496,7 +487,7 @@ set_level_indent(const char *str)
 	  parse_level_string(p, &level_end[1]);
 	  break;
      default:
-	  error(1, 0, _("unknown level indent option: %s"), str);
+	  error(EX_USAGE, 0, _("unknown level indent option: %s"), str);
      }
 }
 
@@ -549,7 +540,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
      case OPT_NO_TREE:
 	  print_as_tree = 0;
 	  level_indent[0] = level_indent[1] = NULL;
-	  level_end[0] = level_end[0] = NULL;
+	  level_end[0] = level_end[1] = NULL;
 	  break;
      case 'b':
 	  brief_listing = 1;
@@ -573,7 +564,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
 	  break;
      case 'f':
 	  if (select_output_driver(arg))
-	       argp_error(state, _("%s: No such output driver"), optarg);
+	       error(EX_USAGE, 0, _("%s: No such output driver"), optarg);
 	  output_init();
 	  break;
      case OPT_LEVEL_INDENT:
@@ -601,7 +592,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
 			 SYMBOL_EXCLUDE(*arg);
 		    break;
 	       default:
-		    argp_error(state, _("Unknown symbol class: %c"), *arg);
+		    error(EX_USAGE, 0, _("Unknown symbol class: %c"), *arg);
 	       }
 	  break;
      case OPT_OMIT_ARGUMENTS:
@@ -728,7 +719,7 @@ include_symbol(Symbol *sym)
 void
 xalloc_die(void)
 {
-     error(1, ENOMEM, _("Exiting"));
+     error(EX_FATAL, ENOMEM, _("Exiting"));
 }
 
 void
@@ -743,7 +734,7 @@ init()
      if (level_end[1] == NULL)
 	  level_end[1] = "";
      
-     init_lex(debug > 1);
+     init_lex(debug > 2);
      init_parse();
 }
 
@@ -751,13 +742,14 @@ const char version_etc_copyright[] =
   /* Do *not* mark this string for translation.  %s is a copyright
      symbol suitable for this locale, and %d is the copyright
      year.  */
-  "Copyright %s 2005, 2006, 2009, 2010, 2011 %d Sergey Poznyakoff";
+  "Copyright %s 2005, 2006, 2009, 2010, 2011, 2014 %d Sergey Poznyakoff";
 
 int
 main(int argc, char **argv)
 {
      int index;
-
+     int status = EX_OK;
+     
      set_program_name(argv[0]);
      argp_version_setup("cflow", program_authors);
      
@@ -771,14 +763,17 @@ main(int argc, char **argv)
      symbol_map = SM_FUNCTIONS|SM_STATIC|SM_UNDEFINED;
 
      if (getenv("POSIXLY_CORRECT")) {
-	  if (select_output_driver("posix"))
-	       error(1, 0, _("%s: No such output driver"), "posix");
+	  if (select_output_driver("posix")) {
+	       error(0, 0, _("INTERNAL ERROR: %s: No such output driver"),
+		     "posix");
+	       abort();
+	  }
 	  output_init();
      }
      
      sourcerc(&argc, &argv);
      if (argp_parse(&argp, argc, argv, ARGP_IN_ORDER, &index, NULL))
-	  exit(1);
+	  exit(EX_USAGE);
 
      if (print_option == 0)
 	  print_option = PRINT_TREE;
@@ -803,13 +798,15 @@ main(int argc, char **argv)
      while (argc--) {
 	  if (source(*argv++) == 0)
 	       yyparse();
+	  else
+	       status = EX_SOFT;
      }
 
      if (input_file_count == 0)
-	     error(1, 0, _("no input files"));
+	     error(EX_USAGE, 0, _("no input files"));
 
      output();
-     return 0;
+     return status;
 }
 
 
